@@ -62,7 +62,7 @@ end
 
     assert_write parser.decls, <<-EOF
 class Hello
-  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped, ?f: ::Integer, **untyped g) { () -> untyped } -> nil
+  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped, ?f: ::Integer, **untyped g) ?{ () -> untyped } -> nil
 
   def self.world: () { (untyped, untyped, untyped, x: untyped, y: untyped) -> untyped } -> untyped
 
@@ -111,6 +111,8 @@ class Hello
   def hash2() { foo: 1 } end
   def hash3() { foo: { bar: 42 }, x: { y: z } } end
   def hash4() { foo: 1, **({ bar: x}).compact } end
+
+  def self1() self end
 end
     EOR
 
@@ -171,6 +173,8 @@ class Hello
   def hash3: () -> { foo: { bar: 42 }, x: { y: untyped } }
 
   def hash4: () -> ::Hash[:foo | untyped, 1 | untyped]
+
+  def self1: () -> self
 end
     EOF
   end
@@ -562,7 +566,7 @@ end
     rb = <<-EOR
 # Comments for class.
 # This is a comment.
-class Hello
+class Hello # :nodoc:
   # Comment for include.
   include Foo
 
@@ -698,6 +702,7 @@ E = nil
 F = false
 G = [1,2,3]
 H = { id: 123 }
+I = self
     EOR
 
     parser.parse(rb)
@@ -718,6 +723,8 @@ F: false
 G: ::Array[1 | 2 | 3]
 
 H: { id: 123 }
+
+I: self
     EOF
   end
 
@@ -883,8 +890,8 @@ end
       [%{{a: nil}}, %{ { a: nil } }],
       [%{{"a" => /b/}}, %{ ::Hash[::String, ::Regexp] }],
     ].each do |rb, rbs|
-      node = RubyVM::AbstractSyntaxTree.parse(rb).children[2]
-      assert_equal RBS::Parser.parse_type(rbs), parser.literal_to_type(node)
+      node = RubyVM::AbstractSyntaxTree.parse("_ = #{rb}").children[2]
+      assert_equal RBS::Parser.parse_type(rbs), parser.literal_to_type(node.children[1])
     end
   end
 
