@@ -6,6 +6,7 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
   Types = RBS::Types
   TypeName = RBS::TypeName
   Namespace = RBS::Namespace
+  AST = RBS::AST
   Declarations = RBS::AST::Declarations
   Members = RBS::AST::Members
   MethodType = RBS::MethodType
@@ -21,17 +22,78 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
 
       assert_instance_of Declarations::Alias, type_decl
       assert_equal TypeName.new(name: :foo, namespace: Namespace.parse("Steep")), type_decl.name
+      assert_equal [], type_decl.type_params.each.map(&:name)
       assert_equal Types::Bases::Any.new(location: nil), type_decl.type
       assert_equal "type Steep::foo = untyped", type_decl.location.source
     end
 
-    assert_raises Parser::SyntaxError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<~RBS)
         type Foo = untyped
       RBS
     end
   end
 
+  def test_type_alias_generic
+    Parser.parse_signature(<<RBS).yield_self do |decls|
+type optional[A] = A?
+RBS
+      assert_equal 1, decls.size
+
+      type_decl = decls[0]
+
+      assert_instance_of Declarations::Alias, type_decl
+      assert_equal TypeName("optional"), type_decl.name
+      assert_equal [:A], type_decl.type_params.each.map(&:name)
+      assert_equal parse_type("A?", variables: [:A]), type_decl.type
+      assert_equal "[A]", type_decl.location[:type_params].source
+    end
+
+    Parser.parse_signature(<<RBS).yield_self do |decls|
+class Foo[A]
+  type bar = B
+end
+RBS
+      decls[0].members[0].tap do |type_decl|
+        assert_instance_of Declarations::Alias, type_decl
+        assert_equal TypeName("bar"), type_decl.name
+        assert_equal [], type_decl.type_params.each.map(&:name)
+        assert_instance_of Types::ClassInstance, type_decl.type
+        assert_nil type_decl.location[:type_params]
+      end
+    end
+  end
+
+  def test_type_alias_generic_variance
+    Parser.parse_signature(<<RBS).yield_self do |decls|
+type x[T] = ^(T) -> void
+
+type y[unchecked out T] = ^(T) -> void
+RBS
+      assert_equal 2, decls.size
+
+      decls[0].tap do |type_decl|
+        assert_instance_of Declarations::Alias, type_decl
+
+        type_decl.type_params[0].tap do |param|
+          assert_equal :T, param.name
+          assert_equal :invariant, param.variance
+          refute_predicate param, :unchecked?
+        end
+      end
+
+      decls[1].tap do |type_decl|
+        assert_instance_of Declarations::Alias, type_decl
+
+        type_decl.type_params[0].tap do |param|
+          assert_equal :T, param.name
+          assert_equal :covariant, param.variance
+          assert_predicate param, :unchecked?
+        end
+      end
+    end
+  end
+
   def test_constant
     Parser.parse_signature("FOO: untyped").yield_self do |decls|
       assert_equal 1, decls.size
@@ -141,7 +203,10 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
           assert_nil t2.block
           assert_equal "(untyped) -> Integer", t2.location.source
 
-          assert_equal [:X], t3.type_params
+          assert_equal(
+            [AST::TypeParam.new(name: :X, variance: :invariant, upper_bound: nil, location: nil)],
+            t3.type_params
+          )
           assert_instance_of Types::Block, t3.block
           assert_instance_of Types::Variable, t3.block.type.required_positionals[0].type
           assert_instance_of Types::Variable, t3.block.type.return_type
@@ -156,7 +221,7 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
       end
     end
 
-    assert_raises Parser::SemanticsError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<~SIG)
         interface _Each[A, B]
           def self.foo: -> void
@@ -164,7 +229,7 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
       SIG
     end
 
-    assert_raises Parser::SemanticsError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<~SIG)
         interface _Each[A, B]
           include Object
@@ -452,27 +517,8 @@ end
     end
   end
 
-  def test_incompatible_method_definition
-    # `incompatible` is ignored with warning message.
-    silence_warnings do
-      Parser.parse_signature(<<~SIG).yield_self do |decls|
-      class Foo
-        incompatible def foo: () -> Integer
-      end
-     SIG
-        assert_equal 1, decls.size
-
-        decls[0].yield_self do |decl|
-          assert_instance_of Declarations::Class, decl
-
-          assert_instance_of Members::MethodDefinition, decl.members[0]
-        end
-      end
-    end
-  end
-
   def test_method_super
-    assert_raises Parser::SyntaxError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<~SIG)
       class Foo
         def foo: -> void
@@ -506,6 +552,108 @@ end
     end
   end
 
+  def test_private_public_def
+    Parser.parse_signature(<<~SIG).yield_self do |decls|
+      class Foo
+        public def foo: () -> void
+        private def bar: () -> void
+        def baz: () -> void
+      end
+    SIG
+
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::Class, decl
+
+        assert_equal 3, decl.members.size
+
+        decl.members[0].tap do |m|
+          assert_instance_of Members::MethodDefinition, m
+          assert_equal :foo, m.name
+          assert_equal :public, m.visibility
+        end
+
+        decl.members[1].tap do |m|
+          assert_instance_of Members::MethodDefinition, m
+          assert_equal :bar, m.name
+          assert_equal :private, m.visibility
+        end
+
+        decl.members[2].tap do |m|
+          assert_instance_of Members::MethodDefinition, m
+          assert_equal :baz, m.name
+          assert_nil m.visibility
+        end
+      end
+    end
+  end
+
+  def test_private_public_def_error
+    assert_raises RBS::ParsingError do
+      Parser.parse_signature(<<~SIG)
+      class Foo
+        public def self?.foo: () -> void
+      end
+    SIG
+    end
+  end
+
+  def test_private_public_attr
+    Parser.parse_signature(<<~SIG).yield_self do |decls|
+      class Foo
+        public attr_reader foo: String
+        private attr_reader bar: String
+        attr_reader baz: String
+      end
+    SIG
+
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::Class, decl
+
+        assert_equal 3, decl.members.size
+
+        decl.members[0].tap do |m|
+          assert_instance_of Members::AttrReader, m
+          assert_equal :foo, m.name
+          assert_equal :public, m.visibility
+        end
+
+        decl.members[1].tap do |m|
+          assert_instance_of Members::AttrReader, m
+          assert_equal :bar, m.name
+          assert_equal :private, m.visibility
+        end
+
+        decl.members[2].tap do |m|
+          assert_instance_of Members::AttrReader, m
+          assert_equal :baz, m.name
+          assert_nil m.visibility
+        end
+      end
+    end
+  end
+
+  def test_private_public_modifier_error
+    assert_raises RBS::ParsingError do
+      Parser.parse_signature(<<~SIG)
+      class Foo
+        public alias foo bar
+      end
+    SIG
+    end
+  end
+
+  def test_private_public_modifier
+    Parser.parse_signature(<<~SIG)
+      class Foo
+        public
+
+        private
+
+        public    # comment can follow
+      end
+    SIG
+  end
+
   def test_alias
     Parser.parse_signature(<<~SIG).yield_self do |decls|
       class Foo
@@ -575,7 +723,7 @@ end
         def attr_reader: -> untyped
         def attr_accessor: -> untyped
         def attr_writer: -> untyped
-        def `\\``: -> untyped
+        def `: -> untyped
         def def!: -> untyped
         def !: -> untyped
         def _foo?: -> untyped
@@ -1017,20 +1165,20 @@ end
       assert_instance_of Declarations::Interface, interface_decl
       a, b, c = interface_decl.type_params.each.to_a
 
-      assert_instance_of Declarations::ModuleTypeParams::TypeParam, a
+      assert_instance_of AST::TypeParam, a
       assert_equal :A, a.name
       assert_equal :invariant, a.variance
-      refute a.skip_validation
+      refute a.unchecked?
 
-      assert_instance_of Declarations::ModuleTypeParams::TypeParam, b
+      assert_instance_of AST::TypeParam, b
       assert_equal :B, b.name
       assert_equal :covariant, b.variance
-      refute b.skip_validation
+      refute b.unchecked?
 
-      assert_instance_of Declarations::ModuleTypeParams::TypeParam, c
+      assert_instance_of AST::TypeParam, c
       assert_equal :C, c.name
       assert_equal :contravariant, c.variance
-      assert c.skip_validation
+      assert c.unchecked?
     end
   end
 
@@ -1096,21 +1244,6 @@ EOF
     end
   end
 
-  def test_overload_def_deprecated
-    silence_warnings do
-      Parser.parse_signature(<<EOF).yield_self do |decls|
-module Steep
-  overload def to_s: (Integer) -> String
-end
-EOF
-        decls[0].members[0].tap do |member|
-          assert_instance_of Members::MethodDefinition, member
-          assert_predicate member, :overload?
-        end
-      end
-    end
-  end
-
   def test_generics_type_parameter
     Parser.parse_signature(<<EOF).yield_self do |decls|
 module A[T]
@@ -1146,14 +1279,14 @@ EOF
   end
 
   def test_syntax_error_on_eof
-    ex = assert_raises Parser::SyntaxError do
+    ex = assert_raises RBS::ParsingError do
       Parser.parse_signature(<<~SIG)
       class Foo
       SIG
     end
-    loc = ex.error_value.location
-    assert_equal 1, loc.start_line
-    assert_equal 9, loc.start_column
+    loc = ex.location
+    assert_equal 2, loc.start_line
+    assert_equal 0, loc.start_column
   end
 
   def test_empty
@@ -1197,7 +1330,7 @@ module A
 end
     EOF
       decls[0].members[0].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "def", foo.location[:keyword].source
         assert_equal "foo", foo.location[:name].source
@@ -1206,7 +1339,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "def", foo.location[:keyword].source
         assert_equal "bar", foo.location[:name].source
@@ -1226,7 +1359,7 @@ module A
 end
     EOF
       decls[0].members[0].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "@foo", foo.location[:name].source
         assert_equal ":", foo.location[:colon].source
@@ -1234,7 +1367,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "@bar", foo.location[:name].source
         assert_equal ":", foo.location[:colon].source
@@ -1242,7 +1375,7 @@ end
       end
 
       decls[0].members[2].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "@@baz", foo.location[:name].source
         assert_equal ":", foo.location[:colon].source
@@ -1264,7 +1397,7 @@ end
     RBS
 
       decls[0].members[0].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader1", foo.location[:name].source
@@ -1275,7 +1408,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader2", foo.location[:name].source
@@ -1286,7 +1419,7 @@ end
       end
 
       decls[0].members[2].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader3", foo.location[:name].source
@@ -1297,7 +1430,7 @@ end
       end
 
       decls[0].members[3].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader4", foo.location[:name].source
@@ -1308,7 +1441,7 @@ end
       end
 
       decls[0].members[4].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader5", foo.location[:name].source
@@ -1319,7 +1452,7 @@ end
       end
 
       decls[0].members[5].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_reader", foo.location[:keyword].source
         assert_equal "reader6", foo.location[:name].source
@@ -1342,7 +1475,7 @@ end
     RBS
 
       decls[0].members[0].tap do |attr|
-        assert_instance_of Location::WithChildren, attr.location
+        assert_instance_of Location, attr.location
 
         assert_equal "attr_writer", attr.location[:keyword].source
         assert_equal "attr1", attr.location[:name].source
@@ -1353,7 +1486,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_writer", foo.location[:keyword].source
         assert_equal "attr2", foo.location[:name].source
@@ -1364,7 +1497,7 @@ end
       end
 
       decls[0].members[2].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_writer", foo.location[:keyword].source
         assert_equal "attr3", foo.location[:name].source
@@ -1375,7 +1508,7 @@ end
       end
 
       decls[0].members[3].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_writer", foo.location[:keyword].source
         assert_equal "attr4", foo.location[:name].source
@@ -1386,7 +1519,7 @@ end
       end
 
       decls[0].members[4].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_writer", foo.location[:keyword].source
         assert_equal "attr5", foo.location[:name].source
@@ -1397,7 +1530,7 @@ end
       end
 
       decls[0].members[5].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_writer", foo.location[:keyword].source
         assert_equal "attr6", foo.location[:name].source
@@ -1420,7 +1553,7 @@ end
     RBS
 
       decls[0].members[0].tap do |attr|
-        assert_instance_of Location::WithChildren, attr.location
+        assert_instance_of Location, attr.location
 
         assert_equal "attr_accessor", attr.location[:keyword].source
         assert_equal "attr1", attr.location[:name].source
@@ -1431,7 +1564,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_accessor", foo.location[:keyword].source
         assert_equal "attr2", foo.location[:name].source
@@ -1442,7 +1575,7 @@ end
       end
 
       decls[0].members[2].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_accessor", foo.location[:keyword].source
         assert_equal "attr3", foo.location[:name].source
@@ -1453,7 +1586,7 @@ end
       end
 
       decls[0].members[3].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_accessor", foo.location[:keyword].source
         assert_equal "attr4", foo.location[:name].source
@@ -1464,7 +1597,7 @@ end
       end
 
       decls[0].members[4].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_accessor", foo.location[:keyword].source
         assert_equal "attr5", foo.location[:name].source
@@ -1475,7 +1608,7 @@ end
       end
 
       decls[0].members[5].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "attr_accessor", foo.location[:keyword].source
         assert_equal "attr6", foo.location[:name].source
@@ -1496,7 +1629,7 @@ end
     RBS
 
       decls[0].members[0].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "alias", foo.location[:keyword].source
         assert_equal "foo", foo.location[:new_name].source
@@ -1506,7 +1639,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "alias", foo.location[:keyword].source
         assert_equal "foo", foo.location[:new_name].source
@@ -1531,7 +1664,7 @@ module A
 end
     EOF
       decls[0].members[0].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "include", foo.location[:keyword].source
         assert_equal "_Foo", foo.location[:name].source
@@ -1539,7 +1672,7 @@ end
       end
 
       decls[0].members[1].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "include", foo.location[:keyword].source
         assert_equal "_Bar", foo.location[:name].source
@@ -1547,7 +1680,7 @@ end
       end
 
       decls[0].members[2].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "extend", foo.location[:keyword].source
         assert_equal "Foo", foo.location[:name].source
@@ -1555,7 +1688,7 @@ end
       end
 
       decls[0].members[3].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "extend", foo.location[:keyword].source
         assert_equal "Bar", foo.location[:name].source
@@ -1563,7 +1696,7 @@ end
       end
 
       decls[0].members[4].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "prepend", foo.location[:keyword].source
         assert_equal "Foo", foo.location[:name].source
@@ -1571,7 +1704,7 @@ end
       end
 
       decls[0].members[5].tap do |foo|
-        assert_instance_of Location::WithChildren, foo.location
+        assert_instance_of Location, foo.location
 
         assert_equal "prepend", foo.location[:keyword].source
         assert_equal "Bar", foo.location[:name].source
@@ -1589,7 +1722,7 @@ interface _B[X, unchecked in Y]
 end
     EOF
       decls[0].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "interface", decl.location[:keyword].source
         assert_equal "_A", decl.location[:name].source
@@ -1598,23 +1731,23 @@ end
       end
 
       decls[1].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "interface", decl.location[:keyword].source
         assert_equal "_B", decl.location[:name].source
         assert_equal "end", decl.location[:end].source
         assert_equal "[X, unchecked in Y]", decl.location[:type_params].source
 
-        decl.type_params[:X].tap do |param|
-          assert_instance_of Location::WithChildren, param.location
+        decl.type_params[0].tap do |param|
+          assert_instance_of Location, param.location
 
           assert_equal "X", param.location[:name].source
           assert_nil param.location[:variance]
           assert_nil param.location[:unchecked]
         end
 
-        decl.type_params[:Y].tap do |param|
-          assert_instance_of Location::WithChildren, param.location
+        decl.type_params[1].tap do |param|
+          assert_instance_of Location, param.location
 
           assert_equal "Y", param.location[:name].source
           assert_equal "in", param.location[:variance].source
@@ -1635,7 +1768,7 @@ end
 module C: BasicObject end
     EOF
       decls[0].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "module", decl.location[:keyword].source
         assert_equal "A", decl.location[:name].source
@@ -1646,7 +1779,7 @@ module C: BasicObject end
       end
 
       decls[1].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "module", decl.location[:keyword].source
         assert_equal "B", decl.location[:name].source
@@ -1657,7 +1790,7 @@ module C: BasicObject end
       end
 
       decls[2].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "module", decl.location[:keyword].source
         assert_equal "C", decl.location[:name].source
@@ -1678,7 +1811,7 @@ class B[X] < Foo[X]
 end
     EOF
       decls[0].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "class", decl.location[:keyword].source
         assert_equal "A", decl.location[:name].source
@@ -1688,7 +1821,7 @@ end
       end
 
       decls[1].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "class", decl.location[:keyword].source
         assert_equal "B", decl.location[:name].source
@@ -1709,21 +1842,21 @@ A::B : String
 $B: Integer
     EOF
       decls[0].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "X", decl.location[:name].source
         assert_equal ":", decl.location[:colon].source
       end
 
       decls[1].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "A::B", decl.location[:name].source
         assert_equal ":", decl.location[:colon].source
       end
 
       decls[2].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "$B", decl.location[:name].source
         assert_equal ":", decl.location[:colon].source
@@ -1736,7 +1869,7 @@ $B: Integer
 type foo = Integer
     EOF
       decls[0].tap do |decl|
-        assert_instance_of Location::WithChildren, decl.location
+        assert_instance_of Location, decl.location
 
         assert_equal "type", decl.location[:keyword].source
         assert_equal "foo", decl.location[:name].source
@@ -1746,7 +1879,7 @@ type foo = Integer
   end
 
   def test_interface_name
-    assert_raises RBS::Parser::SyntaxError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<-RBS)
 interface _foo end
       RBS
@@ -1770,7 +1903,7 @@ end
   end
 
   def test_underscore_type_name
-    assert_raises RBS::Parser::SyntaxError do
+    assert_raises RBS::ParsingError do
       Parser.parse_signature(<<-RBS)
 type _foo = Integer
       RBS
@@ -1778,10 +1911,105 @@ type _foo = Integer
   end
 
   def test_underscore_qualified_name
-    assert_raises RBS::Parser::SyntaxError do
-      Parser.parse_signature(<<-RBS)
+    assert_raises RBS::ParsingError do
+      pp Parser.parse_signature(<<-RBS)
 type x = Foo::_bar
       RBS
     end
   end
+
+  def test_singleton_member_type_variables
+    Parser.parse_signature(<<-EOF).tap do |decls|
+class Foo[A]
+  @foo: A
+
+  @@foo: A
+
+  self.@foo: A
+
+  def foo: -> A
+
+  def self.foo2: -> A
+
+  def self?.foo3: -> A
+
+  attr_accessor self.foo4: A
+
+  attr_reader self.foo5: A
+
+  attr_writer self.foo6: A
+
+  include M[A]
+
+  extend M[A]
+
+  prepend M[A]
+end
+    EOF
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::Class, decl
+
+        assert_instance_of Types::Variable, decl.members[0].type
+        assert_instance_of Types::ClassInstance, decl.members[1].type
+        assert_instance_of Types::ClassInstance, decl.members[2].type
+
+        assert_instance_of Types::Variable, decl.members[3].types[0].type.return_type
+        assert_instance_of Types::ClassInstance, decl.members[4].types[0].type.return_type
+        assert_instance_of Types::ClassInstance, decl.members[5].types[0].type.return_type
+
+        assert_instance_of Types::ClassInstance, decl.members[6].type
+        assert_instance_of Types::ClassInstance, decl.members[7].type
+        assert_instance_of Types::ClassInstance, decl.members[8].type
+
+        assert_instance_of Types::Variable, decl.members[9].args[0]
+        assert_instance_of Types::ClassInstance, decl.members[10].args[0]
+        assert_instance_of Types::Variable, decl.members[11].args[0]
+      end
+    end
+  end
+
+  def test_generics_bound
+    Parser.parse_signature(<<-EOF).tap do |decls|
+class Foo[X < _Each[Y], Y]
+  def foo: [X < Array[Y]] (X) -> X
+end
+    EOF
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::Class, decl
+
+        assert_equal 2, decl.type_params.size
+        decl.type_params[0].tap do |param|
+          assert_equal :X, param.name
+          assert_equal :invariant, param.variance
+          refute_predicate param, :unchecked?
+          assert_equal parse_type("_Each[Y]", variables: [:Y]), param.upper_bound
+        end
+
+        decl.type_params[1].tap do |param|
+          assert_equal :Y, param.name
+          assert_equal :invariant, param.variance
+          refute_predicate param, :unchecked?
+          assert_nil param.upper_bound
+        end
+
+        decl.members[0].tap do |member|
+          member.types[0].type_params[0].tap do |param|
+            assert_equal :X, param.name
+            assert_equal :invariant, param.variance
+            refute_predicate param, :unchecked?
+            assert_equal parse_type("Array[Y]", variables: [:Y]), param.upper_bound
+          end
+        end
+      end
+    end
+  end
+
+  def test_generics_bound_error
+    assert_raises RBS::ParsingError do
+      Parser.parse_signature(<<-EOF)
+        class Foo[X < string]
+        end
+            EOF
+    end
+  end
 end
