@@ -38,3 +38,16 @@ class StringIOTest < StdlibTest
     io.gets(chomp: :true)
   end
 end
+
+class StringIOTypeTest < Test::Unit::TestCase
+  include TypeAssertions
+
+  testing '::StringIO'
+
+  def test_write
+    io = StringIO.new
+
+    assert_send_type "(*String data) -> Integer",
+                     io, :write, "a", "b"
+  end
+end
